@@ -7,15 +7,29 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  */
 
-// Pagination and fuzzy search
+// Pagination and bookmark search
 var url = window.location.pathname;
 var moduleRegex = new RegExp('text\\/(\\w+)\\/');
 var regexArray = moduleRegex.exec(url);
+var userModule = currentModule();
 var modules = ['CALC', 'WRITER', 'IMPRESS', 'DRAW', 'BASE', 'MATH', 'CHART', 'BASIC', 'SHARED'];
 var indexEl = document.getElementsByClassName("index")[0];
-var fullLinks = fullLinkify(indexEl, bookmarks, modules, currentModule());
+var fullLinks = fullLinkify(indexEl, bookmarks, modules, userModule);
 var search = document.getElementById('search-bar');
 search.addEventListener('keyup', debounce(filter, 100, indexEl));
+var flexIndex =  new FlexSearch.Document({ document: {
+        // Only the text content gets indexed, the others get stored as-is
+        index: [{
+            field: 'text',
+            tokenize: 'full'
+        }],
+        store: ['url','app','text']
+    }
+});
+// Populate FlexSearch index
+loadSearch();
+// Render the unfiltered index list on page load
+fillIndex(indexEl, fullLinks, modules);
 // Preserve search input value during the session
 search.value = sessionStorage.getItem('searchsave');
 if (search.value !== undefined) {
@@ -24,20 +38,32 @@ if (search.value !== undefined) {
 window.addEventListener('unload', function(event) {
     sessionStorage.setItem('searchsave', search.value);
 });
-// render the unfiltered index list on page load
-fillIndex(indexEl, fullLinks, modules);
+
+function getQuery(q) {
+   var pattern = new RegExp('[?&]' + q + '=([^&]+)');
+   var param = window.location.search.match(pattern);
+   if (param) {
+       return param[1];
+   }
+   return null;
+}
 
 function currentModule() {
-    var module = '';
-    // get the module name from the URL and remove the first character,
-    // but first deal with snowflake Base
-    if(url.indexOf('explorer/database/') !== -1) {
-        module = 'BASE';
-    } else {
-        if (null === regexArray){// comes from search or elsewhere, no defined module in URL
-            module = 'HARED'
+    // We need to know the module that the user is using when they call for help
+    let module = getQuery('DbPAR');
+    let moduleFromURL = regexArray[1].toUpperCase();
+    if (module == null) {
+        // first deal with snowflake Base
+        if(url.indexOf('/sdatabase/') !== -1) {
+            module = 'BASE';
         } else {
-            module = regexArray[1].toUpperCase().substring(1);
+            if (null === regexArray || moduleFromURL === 'SHARED') {
+                // comes from search or elsewhere, no defined module in URL
+                module = 'SHARED'
+            } else {
+                // drop the 's' from the start
+                module = moduleFromURL.substring(1);
+            }
         }
     }
     return module;
@@ -45,16 +71,21 @@ function currentModule() {
 function fullLinkify(indexEl, bookmarks, modules, currentModule) {
     var fullLinkified = '';
     // if user is not on a shared category page, limit the index to the current module + shared
-    if(currentModule !== 'HARED') {
+    if(currentModule !== 'SHARED') {
         bookmarks = bookmarks.filter(function(obj) {
             return obj['app'] === currentModule || obj['app'] === 'SHARED';
         });
     }
     bookmarks.forEach(function(obj) {
-        fullLinkified += '<a href="' + obj['url'] + '" class="' + obj['app'] + '">' + obj['text'] + '</a>';
+        fullLinkified += '<a href="' + obj['url'] + '" class="' + obj['app'] + '" dir="auto">' + obj['text'] + '</a>';
     });
     return fullLinkified;
 }
+function loadSearch() {
+    bookmarks.forEach((el, i) => {
+        flexIndex.add(i, el);
+    });
+}
 function fillIndex(indexEl, content, modules) {
     indexEl.innerHTML = content;
     var indexKids = indexEl.children;
@@ -72,20 +103,39 @@ function fillIndex(indexEl, content, modules) {
 }
 // filter the index list based on search field input
 function filter(indexList) {
-    var results = null;
-    var target = search.value.trim();
-    var filtered = '';
+    let group = [];
+    let target = search.value.trim();
+    let filtered = '';
     if (target.length < 1) {
         fillIndex(indexEl, fullLinks, modules);
         return;
     }
-    results = fuzzysort.go(target, bookmarks, {threshold: -15000, key:'text'});
+    // Regex for highlighting the match
+    let regex = new RegExp(target.split(/\s+/).filter((i) => i?.length).join("|"), 'gi');
+    let results = flexIndex.search(target, { pluck: "text", enrich: true, limit: 1000 });
 
+    // Similarly to fullLinkify(), limit search results to the user's current module + shared
+    // unless they're somehow not coming from a module.
+    if(userModule !== 'SHARED') {
+        resultModules = [userModule, 'SHARED'];
+    } else {
+        resultModules = modules;
+    }
+
+    // tdf#123506 - Group the filtered list into module groups, keeping the ordering
+    modules.forEach(function(module) {
+        group[module] = '';
+    });
     results.forEach(function(result) {
-        filtered += '<a href="' + result.obj['url'] + '" class="' + result.obj['app'] + '">' + fuzzysort.highlight(result) + '</a>';
+        group[result.doc.app] += '<a href="' + result.doc.url + '" class="' + result.doc.app + '">' + result.doc.text.replace(regex, (match) => `<strong>${match}</strong>`) + '</a>';
+    });
+    resultModules.forEach(function(module) {
+        if (group[module].length > 0) {
+            filtered += group[module];
+        }
     });
-    fillIndex(indexList, filtered, modules);
 
+    fillIndex(indexList, filtered, modules);
 };
 // delay the rendering of the filtered results while user is typing
 function debounce(fn, wait, indexList) {
@@ -98,10 +148,11 @@ function debounce(fn, wait, indexList) {
     };
 }
 
-// copy pycode and bascode to clipboard on mouse click
+// copy pycode, sqlcode and bascode to clipboard on mouse click
 // Show border when copy is done
 divcopyable(document.getElementsByClassName("bascode"));
 divcopyable(document.getElementsByClassName("pycode"));
+divcopyable(document.getElementsByClassName("sqlcode"));
 
 function divcopyable(itemcopyable){
 for (var i = 0, len = itemcopyable.length; i < len; i++) {
@@ -185,4 +236,26 @@ if (typeof linkIndex !== "undefined") {
     }
     current.classList.add('contents-current');
 }
+// close navigation menus when clicking anywhere on the page
+// (ignoring menu button clicks and mobile browsing)
+document.addEventListener('click', function(event) {
+    let a11yButton = event.target.getAttribute("data-a11y-toggle");
+    let vw = Math.max(document.documentElement.clientWidth || 0, window.innerWidth || 0);
+    if (!a11yButton && vw >= 960) {
+        document.querySelectorAll("[data-a11y-toggle] + nav").forEach((el) => {
+            el.setAttribute("aria-hidden", true);
+        });
+    }
+});
+// YouTube consent click. This only works for a single video.
+let youtubePlaceholder = document.querySelector(".youtube_placeholder");
+if (youtubePlaceholder) {
+    youtubePlaceholder.prepend(...document.querySelectorAll(".youtube_consent"));
+}
+function youtubeLoader(ytId, width, height) {
+    let iframeMarkup = `<iframe width="${width}" height="${height}" src="https://www.youtube-nocookie.com/embed/${ytId}?version=3" allowfullscreen="true" frameborder="0"></iframe>`;
+    let placeholder = document.getElementById(ytId);
+    placeholder.innerHTML = iframeMarkup;
+    placeholder.removeAttribute("style");
+}
 /* vim:set shiftwidth=4 softtabstop=4 expandtab cinoptions=b1,g0,N-s cinkeys+=0=break: */
